#include "smball.hxx"
#include <debug.hxx>

#define DEVICE_TYPE ULONG

typedef struct _FILE_FS_VOLUME_INFORMATION {
    LARGE_INTEGER VolumeCreationTime;
    ULONG VolumeSerialNumber;
    ULONG VolumeLabelLength;
    BOOLEAN SupportsObjects;
    CHAR VolumeLabel[1];
//    WCHAR VolumeLabel[1];
} FILE_FS_VOLUME_INFORMATION, *PFILE_FS_VOLUME_INFORMATION;

typedef struct _FILE_FS_SIZE_INFORMATION {
    LARGE_INTEGER TotalAllocationUnits;
    LARGE_INTEGER AvailableAllocationUnits;
    ULONG SectorsPerAllocationUnit;
    ULONG BytesPerSector;
} FILE_FS_SIZE_INFORMATION, *PFILE_FS_SIZE_INFORMATION;

typedef struct _FILE_FS_DEVICE_INFORMATION {                    // ntddk nthal
    DEVICE_TYPE DeviceType;                                     // ntddk nthal
    ULONG Characteristics;                                      // ntddk nthal
} FILE_FS_DEVICE_INFORMATION, *PFILE_FS_DEVICE_INFORMATION;     // ntddk nthal
                                                                // ntddk nthal
typedef struct _FILE_FS_ATTRIBUTE_INFORMATION {
    ULONG FileSystemAttributes;
    LONG MaximumComponentNameLength;
    ULONG FileSystemNameLength;
    CHAR FileSystemName[1];
//    WCHAR FileSystemName[1];
} FILE_FS_ATTRIBUTE_INFORMATION, *PFILE_FS_ATTRIBUTE_INFORMATION;

template<class T>
inline
void
SET_REQ(
    T*& ptr,
    T2B* t2b
    )
{
    ptr = (T*)t2b->in.pParameters;
    t2b->in.pParameters += sizeof(T);
}

inline
void
MORE_ADVANCE_PARAMS(
    T2B* t2b,
    size_t by
    )
{
    t2b->in.pParameters += by;
}

// XXX: we should perhaps worry about padding issues...

inline 
void
ADD_RESP_PARAMS(
    DPB* dpb, 
    T2B* t2b, 
    PVOID resp, 
    size_t size
    )
{
    if (t2b->out.pResp->DataCount) {
        PUCHAR d = (PUCHAR)dpb->out.smb + t2b->out.pResp->DataOffset;
        // need to use MoveMemory due to possible overlap
        RtlMoveMemory((PVOID)(d + size), 
                      (PVOID)d, t2b->out.pResp->DataCount);
        // we up the data offset below
    }
    PUCHAR p = (PUCHAR)dpb->out.smb + t2b->out.pResp->ParameterOffset 
        + t2b->out.pResp->ParameterCount;
    RtlCopyMemory((PVOID)p, (PVOID)resp, (ULONG)size);
    t2b->out.pResp->ParameterCount += size;
    t2b->out.pResp->TotalParameterCount += size;
    *(t2b->out.pByteCount) += size;
    t2b->out.pResp->DataOffset += size;
}

inline 
void
ADD_RESP_DATA(
    DPB* dpb, 
    T2B* t2b, 
    PVOID resp, 
    size_t size
    )
{
    PUCHAR d = (PUCHAR)dpb->out.smb + t2b->out.pResp->DataOffset 
        + t2b->out.pResp->DataCount;
    RtlCopyMemory((PVOID)d, (PVOID)resp, (ULONG)size);
    t2b->out.pResp->DataCount += size;
    t2b->out.pResp->TotalDataCount += size;
    *(t2b->out.pByteCount) += size;
}

////

BOOL
Trans2Unknown(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2Unknown);
    DEBUG_PRINT(("DOSERROR: (------ CANNOT HANDLE THIS TRANS2 FUNCTION ------)\n"));
    SET_DOSERROR(dpb, SERVER, NO_SUPPORT);
    return FALSE;
}

BOOL
Trans2QueryFsInfo(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2QueryFsInfo);
    PREQ_QUERY_FS_INFORMATION pReq;
    SET_REQ(pReq, t2b);

    USHORT uid = dpb->in.smb->Uid;
    FsDispatchTable* pDisp = get_user(uid);
    if (!pDisp) {
        DEBUG_PRINT(("DOSERROR: no such uid\n"));
        SET_DOSERROR(dpb, SERVER, BAD_UID);
        return FALSE;
    }

    fs_attr_t fsattr;
    DWORD error = pDisp->statfs(pDisp->get_root(), &fsattr);
    if (error) {
        DEBUG_PRINT(("WIN32ERROR: statfs error 0x%08X\n", error));
        SET_WIN32ERROR(dpb, error);
        return FALSE;
    }

    switch (pReq->InformationLevel) {
    case SMB_INFO_ALLOCATION: {
        FSALLOCATE Resp;
        Resp.idFileSystem = 0;
        Resp.cSectorUnit = 1;
        Resp.cbSector = 0x0400; // 1024-byte sectors
        Resp.cUnit = fsattr.total_bytes/(Resp.cSectorUnit*Resp.cbSector);
        Resp.cUnitAvail = fsattr.free_bytes/(Resp.cSectorUnit*Resp.cbSector);
        ADD_RESP_DATA(dpb, t2b, &Resp, sizeof(FSALLOCATE));
        return TRUE;
    }
    case SMB_INFO_VOLUME: {
        FSINFO Resp;
        Resp.ulVsn = 0;
        const size_t maxlen = sizeof(Resp.vol.szVolLabel);
        Resp.vol.cch = min(lstrlen(fsattr.fs_name), maxlen);
        lstrcpyn(Resp.vol.szVolLabel, fsattr.fs_name, maxlen);
        ADD_RESP_DATA(dpb, t2b, &Resp, sizeof(FSINFO));
        return TRUE;
    }
    case SMB_QUERY_FS_VOLUME_INFO: {
        const int name_len = lstrlen(fsattr.fs_name);
        const size_t size = sizeof(FILE_FS_VOLUME_INFORMATION) + name_len;
        PFILE_FS_VOLUME_INFORMATION pResp = 
            (PFILE_FS_VOLUME_INFORMATION)new UCHAR[size];
        pResp->VolumeCreationTime.QuadPart = 0;
        pResp->VolumeSerialNumber = 0;
        pResp->VolumeLabelLength = name_len * sizeof(WCHAR);
        wsprintf((char*)pResp->VolumeLabel, "%S", fsattr.fs_name);
        ADD_RESP_DATA(dpb, t2b, pResp, size);
        delete[] (PUCHAR)pResp;
        return TRUE;
    }
    default:
        DEBUG_PRINT(("DOSERROR: <COULD NOT UNDERSTAND INFO LEVEL>\n"));
        SET_DOSERROR(dpb, SERVER, NO_SUPPORT);
        return FALSE;
    }
}

typedef struct _FILE_DIRECTORY_INFORMATION {
    ULONG NextEntryOffset;
    ULONG FileIndex;
    LARGE_INTEGER CreationTime;
    LARGE_INTEGER LastAccessTime;
    LARGE_INTEGER LastWriteTime;
    LARGE_INTEGER ChangeTime;
    LARGE_INTEGER EndOfFile;
    LARGE_INTEGER AllocationSize;
    ULONG FileAttributes;
    ULONG FileNameLength;
    CHAR FileName[1];
//    WCHAR FileName[1];
} FILE_DIRECTORY_INFORMATION, *PFILE_DIRECTORY_INFORMATION;

typedef struct _FILE_FULL_DIR_INFORMATION {
    ULONG NextEntryOffset;
    ULONG FileIndex;
    LARGE_INTEGER CreationTime;
    LARGE_INTEGER LastAccessTime;
    LARGE_INTEGER LastWriteTime;
    LARGE_INTEGER ChangeTime;
    LARGE_INTEGER EndOfFile;
    LARGE_INTEGER AllocationSize;
    ULONG FileAttributes;
    ULONG FileNameLength;
    ULONG EaSize;
    CHAR FileName[1];
//    WCHAR FileName[1];
} FILE_FULL_DIR_INFORMATION, *PFILE_FULL_DIR_INFORMATION;

typedef struct _FILE_BOTH_DIR_INFORMATION {
    ULONG NextEntryOffset;
    ULONG FileIndex;
    LARGE_INTEGER CreationTime;
    LARGE_INTEGER LastAccessTime;
    LARGE_INTEGER LastWriteTime;
    LARGE_INTEGER ChangeTime;
    LARGE_INTEGER EndOfFile;
    LARGE_INTEGER AllocationSize;
    ULONG FileAttributes;
    ULONG FileNameLength;
    ULONG EaSize;
    CCHAR ShortNameLength;
    WCHAR ShortName[12];
    CHAR FileName[1];
//    WCHAR FileName[1];
} FILE_BOTH_DIR_INFORMATION, *PFILE_BOTH_DIR_INFORMATION;

typedef struct _FILE_NAMES_INFORMATION {
    ULONG NextEntryOffset;
    ULONG FileIndex;
    ULONG FileNameLength;
    CHAR FileName[1];
//    WCHAR FileName[1];
} FILE_NAMES_INFORMATION, *PFILE_NAMES_INFORMATION;

const char*
find_filter(
    const char* filename
    )
{
    const char* p = filename + lstrlen(filename);
    while ((p >= filename) && (*p != '\\') && (*p != '/')) p--;
    if (p >= filename) return ++p;
    return 0;
}

bool
get_dir_entries(
    FsDispatchTable* pDisp,
    fhandle_t h,
    UINT32& cookie,
    const filter_t& filter,
    USHORT info_level,
    bool add_res_key,
    USHORT max_count,
    USHORT& count,
    bool& done,
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(get_dir_entries);
    int space_left = dpb->out.size - dpb->out.valid;
    int base_size;

    switch (info_level) {
        case SMB_INFO_STANDARD:
            base_size = sizeof(SMB_FIND_BUFFER);
            break;
        case SMB_INFO_QUERY_EA_SIZE:
            base_size = sizeof(SMB_FIND_BUFFER2);
            break;
        case SMB_INFO_QUERY_EAS_FROM_LIST:
        case SMB_FIND_FILE_DIRECTORY_INFO:
        case SMB_FIND_FILE_FULL_DIRECTORY_INFO:
        case SMB_FIND_FILE_NAMES_INFO:
        case SMB_FIND_FILE_BOTH_DIRECTORY_INFO:
        default:
            DEBUG_PRINT(("DOSERROR: <COULD NOT UNDERSTAND INFO LEVEL>\n"));
            SET_DOSERROR(dpb, SERVER, NO_SUPPORT);
            return false;
    }

    done = false;

    UINT32 sub_count = 0;
    dirinfo_t entry;
    DWORD name_len, len;
    DWORD max_entry_size = base_size + sizeof(entry.name) + 
        (add_res_key?sizeof(ULONG):0);

    while ((count < max_count) && (space_left >= max_entry_size) && !done) {

        DWORD error = pDisp->read_dir(h, cookie, &entry, sizeof(entry), &sub_count);
        done = (error == ERROR_NO_MORE_FILES);

        if (done) return true;

        if (sub_count != 1) {
            DEBUG_PRINT(("DOSERROR: count is %d, error is 0x%08X, sub_count is %d\n",
                         count, error, sub_count));
            SET_DOSERROR(dpb, SERVER, ERROR); // INTERNAL ERROR
            return false;
        }

        cookie = entry.cookie;

        if (!filter.match(entry.name)) continue;

        count++;

        if (add_res_key) {
            ULONG res_key = entry.cookie;
            ADD_RESP_DATA(dpb, t2b, &res_key, sizeof(res_key));
            space_left -= sizeof(res_key);
        }
        name_len = lstrlen(entry.name);
        len = base_size + name_len;
        space_left -= len;
        
        switch (info_level) {
        case SMB_INFO_STANDARD: {
            PSMB_FIND_BUFFER pResp = (PSMB_FIND_BUFFER) new UCHAR[len];
            time64_to_smb_datetime(entry.attribs.create_time,
                                   pResp->CreationDate.Ushort,
                                   pResp->CreationTime.Ushort);
            time64_to_smb_datetime(entry.attribs.access_time,
                                   pResp->LastAccessDate.Ushort,
                                   pResp->LastAccessTime.Ushort);
            time64_to_smb_datetime(entry.attribs.mod_time,
                                   pResp->LastWriteDate.Ushort,
                                   pResp->LastWriteTime.Ushort);
            pResp->DataSize = entry.attribs.file_size;
            pResp->AllocationSize = entry.attribs.alloc_size;
            pResp->Attributes = attribs_to_smb_attribs(entry.attribs.attributes);
            pResp->FileNameLength = name_len;
            lstrcpy(pResp->FileName, entry.name);
            ADD_RESP_DATA(dpb, t2b, pResp, len);
            delete [] (PUCHAR)pResp;
            break;
        }
        case SMB_INFO_QUERY_EA_SIZE: {
            PSMB_FIND_BUFFER2 pResp = (PSMB_FIND_BUFFER2) new UCHAR[len];
            time64_to_smb_datetime(entry.attribs.create_time,
                                   pResp->CreationDate.Ushort,
                                   pResp->CreationTime.Ushort);
            time64_to_smb_datetime(entry.attribs.access_time,
                                   pResp->LastAccessDate.Ushort,
                                   pResp->LastAccessTime.Ushort);
            time64_to_smb_datetime(entry.attribs.mod_time,
                                   pResp->LastWriteDate.Ushort,
                                   pResp->LastWriteTime.Ushort);
            pResp->DataSize = entry.attribs.file_size;
            pResp->AllocationSize = entry.attribs.alloc_size;
            pResp->Attributes = attribs_to_smb_attribs(entry.attribs.attributes);
            pResp->EaSize = 0;
            pResp->FileNameLength = name_len;
            lstrcpy(pResp->FileName, entry.name);
            ADD_RESP_DATA(dpb, t2b, pResp, len);
            delete [] (PUCHAR)pResp;
            break;
        }
        case SMB_INFO_QUERY_EAS_FROM_LIST:
        case SMB_FIND_FILE_DIRECTORY_INFO:
        case SMB_FIND_FILE_FULL_DIRECTORY_INFO:
        case SMB_FIND_FILE_NAMES_INFO:
        case SMB_FIND_FILE_BOTH_DIRECTORY_INFO:
        default: {
            DEBUG_PRINT(("DOSERROR: <COULD NOT UNDERSTAND INFO LEVEL>\n"));
            SET_DOSERROR(dpb, SERVER, NO_SUPPORT);
            return false;
        }
        }
    } // while...
    return true;
}

BOOL
Trans2FindFirst2(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2FindFirst2);
    PREQ_FIND_FIRST2 pReq;
    SET_REQ(pReq, t2b);

    USHORT uid = dpb->in.smb->Uid;
    FsDispatchTable* pDisp = get_user(uid);
    if (!pDisp) {
        DEBUG_PRINT(("DOSERROR: could not find dispatch table for uid %d\n", uid));
        SET_DOSERROR(dpb, SERVER, BAD_UID);
        return FALSE;
    }

    HACKNAME(pReq->Buffer);
    const char* filter_str = find_filter((const char*)pReq->Buffer);
    if (!filter_str) {
        DEBUG_PRINT(("DOSERROR: could not find filter in %s\n",pReq->Buffer));
        SET_DOSERROR(dpb, SERVER, FILE_SPECS);
        return FALSE;
    }

    int path_len = filter_str - (const char*)pReq->Buffer + 1;
    char* path = new char[path_len];
    lstrcpyn(path, (const char*)pReq->Buffer, path_len);

    fhandle_t h;
    DWORD error = pDisp->create(pDisp->get_root(),
                                path,
                                DISP_DIRECTORY | SHARE_READ | SHARE_WRITE,
                                0,
                                &h);
    // NOTE: delete [] path below due to debug statement below...
    if (error) {
        DEBUG_PRINT(("WIN32ERROR: could not create %s (0x%08X)\n",path,error));
        SET_WIN32ERROR(dpb, error);
        delete [] path;
        return FALSE;
    }
    delete [] path;

    USHORT sid = add_find(h, filter_str);
    const filter_t* pfilter = 0;
    if (!get_find(sid, 0, &pfilter, 0) || !pfilter) {
        DEBUG_PRINT(("DOSERROR: could not get filter back!\n"));
        SET_DOSERROR(dpb, SERVER, ERROR); // INTERNAL ERROR
        return FALSE;
    }


    RESP_FIND_FIRST2 Resp;
    Resp.Sid = sid;
    Resp.SearchCount = 0;
    Resp.EndOfSearch = 0;
    Resp.EaErrorOffset = 0;
    Resp.LastNameOffset = 0;


    DEBUG_PRINT(("FLAGS: 0x%04x\n", pReq->Flags));

    UINT32 cookie = 0;
    bool done = false;

    if (get_dir_entries(pDisp, h, cookie, *pfilter, pReq->InformationLevel,
                        pReq->Flags & SMB_FIND_RETURN_RESUME_KEYS, 
                        pReq->SearchCount, Resp.SearchCount,
                        done, dpb, t2b)) {
        save_find_cookie(Resp.Sid, cookie); // XXX - no error-checking
        if (done) Resp.EndOfSearch = 1;

        if ((pReq->Flags & SMB_FIND_CLOSE_AFTER_REQUEST) || 
            ((pReq->Flags & SMB_FIND_CLOSE_AT_EOS) && done)) {
            Resp.Sid = 0xFFFF;
        } else if (!done) {
            // XXX - offset into last entry if resume needed...
        }

        ADD_RESP_PARAMS(dpb, t2b, &Resp, sizeof(RESP_FIND_FIRST2));
    }
    if ((pReq->Flags & SMB_FIND_CLOSE_AFTER_REQUEST) || 
        ((pReq->Flags & SMB_FIND_CLOSE_AT_EOS) && done)) {
        pDisp->close(h);
        del_find(sid);
    }
    return TRUE;
}

BOOL
Trans2FindNext2(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2FindNext2);
    PREQ_FIND_NEXT2 pReq;
    SET_REQ(pReq, t2b);

    USHORT uid = dpb->in.smb->Uid;
    FsDispatchTable* pDisp = get_user(uid);
    if (!pDisp) {
        DEBUG_PRINT(("DOSERROR: no such uid\n"));
        SET_DOSERROR(dpb, SERVER, BAD_UID);
        return FALSE;
    }

    fhandle_t h;
    const filter_t* pfilter = 0;
    UINT32 cookie;

    if (!get_find(pReq->Sid, &h, &pfilter, &cookie) || !pfilter) {
        DEBUG_PRINT(("DOSERROR: could not find sid\n"));
        SET_DOSERROR(dpb, DOS, BAD_FID);
        return FALSE;
    }

    RESP_FIND_NEXT2 Resp;
    Resp.SearchCount = 0;
    Resp.EndOfSearch = 0;
    Resp.EaErrorOffset = 0;
    Resp.LastNameOffset = 0;

    if (!(pReq->Flags & SMB_FIND_CONTINUE_FROM_LAST)) cookie = pReq->ResumeKey;

    DEBUG_PRINT(("FLAGS: 0x%04x\n", pReq->Flags));

    bool done = false;

    if (get_dir_entries(pDisp, h, cookie, *pfilter, pReq->InformationLevel,
                        pReq->Flags & SMB_FIND_RETURN_RESUME_KEYS, 
                        pReq->SearchCount, Resp.SearchCount,
                        done, dpb, t2b)) {
        save_find_cookie(pReq->Sid, cookie); // XXX - no error-checking
        if (done) Resp.EndOfSearch = 1;
        // XXX - offset into last entry if resume needed...
        ADD_RESP_PARAMS(dpb, t2b, &Resp, sizeof(RESP_FIND_NEXT2));
    }
    if ((pReq->Flags & SMB_FIND_CLOSE_AFTER_REQUEST) || 
        ((pReq->Flags & SMB_FIND_CLOSE_AT_EOS) && done)) {
        pDisp->close(h);
        del_find(pReq->Sid);
    }
    return TRUE;
}

BOOL
Trans2QueryPathInfo(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2QueryPathInfo);
    PREQ_QUERY_PATH_INFORMATION pReq;
    SET_REQ(pReq, t2b);

    USHORT uid = dpb->in.smb->Uid;
    FsDispatchTable* pDisp = get_user(uid);
    if (!pDisp) {
        DEBUG_PRINT(("DOSERROR: no such uid\n"));
        SET_DOSERROR(dpb, SERVER, BAD_UID);
        return FALSE;
    }

    LPCSTR file_name = (LPCSTR)pReq->Buffer;
    HACKNAME(file_name);

    fattr_t attribs;
    DWORD error = pDisp->lookup(pDisp->get_root(), file_name, &attribs);
    if (error) {
        DEBUG_PRINT(("WIN32ERROR: lookup error 0x%08X\n", error));
        SET_WIN32ERROR(dpb, error);
        return FALSE;
    }

    switch(pReq->InformationLevel) {
    case SMB_INFO_STANDARD: {
        DWORD len = sizeof(SMB_FIND_BUFFER) - sizeof(UCHAR) - sizeof(CHAR);
        PSMB_FIND_BUFFER pResp = (PSMB_FIND_BUFFER) new UCHAR[len];
        time64_to_smb_datetime(attribs.create_time,
                               pResp->CreationDate.Ushort,
                               pResp->CreationTime.Ushort);
        time64_to_smb_datetime(attribs.access_time,
                               pResp->LastAccessDate.Ushort,
                               pResp->LastAccessTime.Ushort);
        time64_to_smb_datetime(attribs.mod_time,
                               pResp->LastWriteDate.Ushort,
                               pResp->LastWriteTime.Ushort);
        pResp->DataSize = attribs.file_size;
        pResp->AllocationSize = attribs.alloc_size;
        pResp->Attributes = attribs_to_smb_attribs(attribs.attributes);
        ADD_RESP_DATA(dpb, t2b, pResp, len);
        delete [] (PUCHAR)pResp;
        return TRUE;
    }
    case SMB_INFO_QUERY_EA_SIZE: {
        DWORD len = sizeof(SMB_FIND_BUFFER2) - sizeof(UCHAR) - sizeof(CHAR);
        PSMB_FIND_BUFFER2 pResp = (PSMB_FIND_BUFFER2)new UCHAR[len];
        time64_to_smb_datetime(attribs.create_time,
                               pResp->CreationDate.Ushort,
                               pResp->CreationTime.Ushort);
        time64_to_smb_datetime(attribs.access_time,
                               pResp->LastAccessDate.Ushort,
                               pResp->LastAccessTime.Ushort);
        time64_to_smb_datetime(attribs.mod_time,
                               pResp->LastWriteDate.Ushort,
                               pResp->LastWriteTime.Ushort);
        pResp->DataSize = attribs.file_size;
        pResp->AllocationSize = attribs.alloc_size;
        pResp->Attributes = attribs_to_smb_attribs(attribs.attributes);
        pResp->EaSize = 0;
        ADD_RESP_DATA(dpb, t2b, pResp, len);
        delete [] (PUCHAR)pResp;
        return TRUE;
    }
    case SMB_INFO_QUERY_EAS_FROM_LIST:
    case SMB_FIND_FILE_DIRECTORY_INFO:
    case SMB_FIND_FILE_FULL_DIRECTORY_INFO:
    case SMB_FIND_FILE_NAMES_INFO:
    case SMB_FIND_FILE_BOTH_DIRECTORY_INFO:
    default: {
        DEBUG_PRINT(("DOSERROR: <COULD NOT UNDERSTAND INFO LEVEL>\n"));
        SET_DOSERROR(dpb, SERVER, NO_SUPPORT);
        return FALSE;
    }
    }
}

BOOL
Trans2SetPathInfo(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2SetPathInfo);
    return Trans2Unknown(dpb, t2b);
}

BOOL
Trans2QueryFileInfo(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2QueryFileInfo);
    return Trans2Unknown(dpb, t2b);
}

BOOL
Trans2SetFileInfo(
    DPB* dpb,
    T2B* t2b
    )
{
    DFN(Trans2SetFileInfo);
    return Trans2Unknown(dpb, t2b);
}
